/***************************************************************************
 *   Copyright (C) 2004 by Nikola Kotur                                    *
 *   kotnik@ns-linux.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qstring.h>
#include <qdir.h>
#include <qfile.h>
#include <qtextstream.h>

#include <cstdlib>

#include "mcconf.h"


McConf::McConf()
{
	home_dir = path = std::getenv("HOME");
	path += "/.nkrecnik";
	progDirPath = path;
	QDir s_dir(path);
	QDir h_dir(home_dir);
	path += "/nkrecnik.conf";
	if (!s_dir.exists())
		h_dir.mkdir(".nkrecnik", true);
	setFile.setName(path); // setFile is QFile of settings file.
	// Load defaults:
	loadDefaults();
	loadSettings();
}

McConf::~McConf()
{
	saveSettings();
}


QString McConf::progDir()
{
	return progDirPath;
}


QString McConf::homeDir()
{
	return home_dir;
}

// Sets default values.
void McConf::loadDefaults()
{
	win_x = win_y = 0;
	tab_left = tab_right = 27;
	clip_store_trans = 1;
	clip_store_format = 0;
	clip_format = "%1 %2";
	add_dict = "";
	kde_mode = 0;
}


void McConf::loadSettings()
{
	QString line;
	if (setFile.open(IO_ReadOnly))
	{
		QTextStream stream(&setFile);

		if (!stream.atEnd())
		{
			line = stream.readLine();
			if (line == "NKR-0.8")
			{
				QString entry;
				bool ok;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				win_x = line.toInt(&ok, 10);
				if (!ok) win_x = 0;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				win_y = line.toInt(&ok, 10);
				if (!ok) win_y = 0;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				tab_left = line.toInt(&ok, 10);
				if (!ok) tab_left = 27;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				tab_right = line.toInt(&ok, 10);
				if (!ok) tab_right = 27;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				clip_store_trans = line.toInt(&ok, 10);
				if (!ok) clip_store_trans = 1;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				clip_store_format = line.toInt(&ok, 10);
				if (!ok) clip_store_format = 0;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				clip_format = line;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				add_dict = line;

				if (!stream.atEnd()) line = stream.readLine();
				else return;
				kde_mode = line.toInt(&ok, 10);
				if (!ok) kde_mode = 0;
			}
			else
			{
				oldVersion();
			}
		}
		setFile.close();
	}
}


void McConf::saveSettings()
{
	QString line;
	if (setFile.open(IO_WriteOnly))
	{
		QTextStream stream(&setFile);

		stream << "NKR-0.8" << endl;

		line.setNum(win_x);
		stream << line << endl;

		line.setNum(win_y);
		stream << line << endl;

		line.setNum(tab_left);
		stream << tab_left << endl;

		line.setNum(tab_right);
		stream << tab_right << endl;

		line.setNum(clip_store_trans);
		stream << line << endl;

		line.setNum(clip_store_format);
		stream << line << endl;

		stream << clip_format << endl;

		stream << add_dict << endl;

		line.setNum(kde_mode);
		stream << line << endl;

		setFile.close();
	}
}

void McConf::oldVersion()
{
}
