/***************************************************************************
 *   Copyright (C) 2004 by Nikola Kotur                                    *
 *   kotnik@ns-linux.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kstddirs.h>

#include <qstring.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qtextcodec.h>
#include <cstring>
#include <iostream>
#include <stdlib.h>
#include <stdio.h>

#include "mcdict.h"


McDict::McDict()
{
	dictPath = "";
	init();
}


McDict::McDict(QString place, int style)
{
	switch (style)
	{
		case KDE_STYLE:
			dirs = KGlobal::dirs();
			dictPath = dirs->findResource("data", "nkrecnik/data/" + place);
//			qDebug("dictPath= " + dictPath);
			break;
		case PLAIN_STYLE:
			dictPath = place;
			break;
	}
	init();
}


McDict::~McDict()
{
}


QString McDict::whereIsDict()
{
	return dictPath;
}


void McDict::init()
{
	usedDict = EN2SR; // 0 - sr2en, 1 - en2sr
				   // 1 means: QUERY | RESULT
				   // 0 means: RESULT| QUERY
	clearWords();
	doSort = true;
}


bool McDict::doesExist()
{
	QFile dictionary(dictPath);
	return dictionary.exists();
}


/**
	Locate translations in dictionary, returns
	number of found translations.
 */
int McDict::find(QString query)
{
	int count=0;
	if (!(query.isEmpty()))
	{
		clearWords();
		QFile dictionary(dictPath);
		if (dictionary.open(IO_ReadOnly))
		{
			QTextStream stream(&dictionary);
			QString line, word1, word2;
			while (!stream.atEnd())
			{
				line = stream.readLine(); // Without \n
				if (usedDict == EN2SR)
					parseWords(&line, &word1, &word2);
				else
					parseWords(&line, &word2, &word1);
				word1 = word1.lower();
				if (isHere(&word1, &query))
				{
					putWords(word1, word2, count);
					++count;
				}
			}
			dictionary.close();
			if (doSort) sortWords(count);
		}
	}
	return count;
}


void McDict::clearWords()
{
	for (int i = 0; i < 100; i++)
	{
		firstWords[i] = "";
		secondWords[i] = "";
		wordCount[i] = 0;
	}
}


void McDict::parseWords(QString *line, QString *first, QString *second)
{
	// This is very slow, and it can't be done otherwise:
	//*first = line->section('|', 0, 0);
	//*second = line->section('|', 1, 1);
	strcpy (cLine, line->latin1());
	strcpy (cFirst, "");
	strcpy (cSecond, "");

	unsigned int i = 0, j = 0;
	int limiter = '|';
	do
	{
		if (cLine[i] != limiter)
		{
			cFirst[j] = cLine[i];
			j++;
		}
		else break;
		i++;
	} while (i < strlen(cLine));
	cFirst[j] = '\0';
	i++;
	j = 0;
	do
	{
		if (cLine[i] != limiter)
		{
			cSecond[j] = cLine[i];
			j++;
		}
		else break;
		i++;
	} while (i < strlen(cLine));
	cSecond[j] = '\0';

	*first = cFirst;
	*second = cSecond;
}


bool McDict::isHere(QString *dictWord, QString *userWord)
{
	if (dictWord->contains(*userWord) > 0)
	{
		// Is word alone
		if (dictWord->length() == userWord->length())
			return true;

		// Does it end with it
		if (dictWord->endsWith(" " + *userWord))
			return true;

		// Does it start with it
		if (dictWord->startsWith(*userWord + " "))
			return true;

		// Is it in the middle
		if (dictWord->contains(" " + *userWord + " "))
			return true;
	}
	return false;
}


void McDict::putWords(QString first, QString second, int index)
{
	if (index < 100)
	{
		firstWords[index] = first;
		secondWords[index] = second;
	}
}


void McDict::sortWords(int num)
{
	countWords(num);

	// Sort with number of words
	for (int i = 0; i < num; i++)
		for (int j = i + 1; j < num; j++)
			if (wordCount[i] > wordCount[j])
				swapWords(i, j);

	// Sort alphabetically
	for (int i = 0; i < num; i++)
	{
		for (int j = i + 1; j < num; j++)
		{
			if (wordCount[i] == wordCount[j])
			{
				if (firstWords[i] > firstWords[j])
					swapWords(i, j);
				else
					if (secondWords[i] > secondWords[j])
						swapWords(i, j);
			}
		}
	}
}


void McDict::swapWords(int index1, int index2)
{
	QString tempS;
	int tempI;
	tempS = firstWords[index1];
	tempI = wordCount[index1];
	firstWords[index1] = firstWords[index2];
	wordCount[index1] = wordCount[index2];
	firstWords[index2] = tempS;
	wordCount[index2] = tempI;
	tempS = secondWords[index1];
	secondWords[index1] = secondWords[index2];
	secondWords[index2] = tempS;
}


QString McDict::getWord(int index, int column)
{
	if (index < 100)
	{
		QTextCodec *codec = QTextCodec::codecForName("utf8");
		if (column)
			return codec->toUnicode(secondWords[index]);
		else
			return codec->toUnicode(firstWords[index]);
	}
	else return "";
}


void McDict::dictChange(int dict)
{
	usedDict = dict;
}


void McDict::countWords(int num)
{
	int spaces;
	for (int i = 0; i < num; i++)
	{
		spaces = 0;
		for (unsigned int j = 0; j < firstWords[i].length(); j++)
			if (firstWords[i][j] == ' ')
				++spaces;
		wordCount[i] = ++spaces;
	}
}


void McDict::setSort(bool method)
{
	doSort = method;
}


bool McDict::writeEntry(QString first, QString second)
{
	QFile dict(dictPath);
	if (dict.open(IO_WriteOnly | IO_Append))
	{
		QString line(first+"|"+second+"|");
		QTextStream stream(&dict);
		stream << line << endl;
		dict.close();
		return true;
	}
	else
		return false;
}
