/***************************************************************************
 *   Copyright (C) 2004 by Nikola Kotur                                    *
 *   kotnik@ns-linux.org                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <klocale.h>
#include <kfiledialog.h>

#include <qlabel.h>
#include <qmessagebox.h>
#include <qradiobutton.h>
#include <qheader.h>
#include <qsize.h>
#include <qnamespace.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qcursor.h>
#include <qlineedit.h>
#include <qtable.h>
#include <qtabwidget.h>
#include <qtextedit.h>
#include <qcheckbox.h>
#include <qbutton.h>
#include <qspinbox.h>

#include "nkrecnik.h"
#include "nkrGUI.h"
#include "mcdict.h"
#include "mcconf.h"


NKRecnik::NKRecnik(QWidget *parent, const char *name) : nkrGUI(parent, name)
{
	logger->append("Starting...");

	// Loading configuration file
	config = new McConf();
	logger->append("Loading configuration file");

	// Loading default dictionary
	Dict = new McDict("dict.dat", KDE_STYLE);
// 	kdeDict = new McDict("kde-dict.dat", KDE_STYLE);
	logger->append("Default dictionary loaded");

	// Loading user dictionary
 	QString updictfile = config->progDir() + "/updictionary.dat";
 	updateDict = new McDict(updictfile, PLAIN_STYLE);
 	logger->append("User dictionary loaded, its: " + updictfile);

	// Loading additional dictionary.
	if (config->add_dict != "")
	{
		additionalDict = new McDict(config->add_dict, PLAIN_STYLE);
		if (additionalDict->doesExist())
			addDictLoaded = true;
		else
		{
			addDictLoaded = false;
			delete additionalDict;
		}
	}
	else
		addDictLoaded = false;

	// Making clipboard access.
	clipBoard = QApplication::clipboard();

	// Disabling sorting
	Dict->setSort(false);
// 	updateDict->setSort(false);

	// Setting some options from config file (others in constructor)
	logger->append("Setting configuration");
	if (config->clip_store_trans)
	{
		clipTrans->setChecked(true);
		clipFormat->setChecked(false);
	}
	else
	{
		clipTrans->setChecked(false);
		clipFormat->setChecked(true);
	}
	lineFormat->setText(config->clip_format);

	lineEdit->setFocus();	// Gives focus to line edit
	//this->setFocus();

	this->setGeometry(config->win_x, config->win_y, this->width(), this->height());

	// Clear table
	for (int i=0; i < 100; i++)
		table->hideRow(i);

	// Making table look.
	table->horizontalHeader()->setLabel(0, tr2i18n("English"));		// Labels...
	table->horizontalHeader()->setLabel(1, tr2i18n("Serbian"));
	tabSize = table->size();
	table->setColumnWidth(0, (tabSize.width() / 2) - config->tab_left);	// Coulmn widths... (from config file)
	table->setColumnWidth(1, (tabSize.width() / 2) - config->tab_right);

	// Some additional variables;
	curRow = curCol = 0;	// clearing variables for table clicked slot.
	logger->append("Started properly");
}


// @todo Destructor problem.  It just isn't called!
NKRecnik::~NKRecnik()
{
	delete config;
	delete Dict;
// 	delete kdeDict;
// 	delete updateDict;
// 	delete additionalDict;
}


/**
 * Called upon enter pressed, or find clicked
 */
void NKRecnik::retPressed()
{
	//QTime t;
	//t.start();
	tabs->setCurrentPage(0);
	numAllWords = 0;
	for (int i=0; i < 100; i++)
	{
		table->setText(i, 0, "");
		table->setText(i, 1, "");
		table->hideRow(i);
	}

	QString queryPhrase(lineEdit->displayText());
	queryPhrase = queryPhrase.lower();

	logger->append("Looking for " + queryPhrase + ". Found:");
	int upNumWords=0, numWords=0, adNumWords=0;

// 	if (!inKdeMode)
// 	{
		numWords = Dict->find(queryPhrase);
		if (updateDict->doesExist())
			upNumWords = updateDict->find(queryPhrase);
		if (addDictLoaded)
			adNumWords = additionalDict->find(queryPhrase);

		// collect all words from all the dictionaries
		for (int i = 0; i < numWords; i++)
		{
			allWords[numAllWords][0] = Dict->getWord(i, 0);
			allWords[numAllWords][1] = Dict->getWord(i, 1);
			++numAllWords;
		}
		for (int i = 0; i < upNumWords; i++)
		{
			allWords[numAllWords][0] = updateDict->getWord(i, 0);
			allWords[numAllWords][1] = updateDict->getWord(i, 1);
			++numAllWords;
		}
		for (int i = 0; i < adNumWords; i++)
		{
			allWords[numAllWords][0] = additionalDict->getWord(i, 0);
			allWords[numAllWords][1] = additionalDict->getWord(i, 1);
			++numAllWords;
		}
/* 	}
	else // So, we're in KDE mode.
 	{
 		numWords = kdeDict->find(queryPhrase);
		for (int i = 0; i < numWords; i++)
		{
			allWords[numAllWords][0] = kdeDict->getWord(i, 0);
			allWords[numAllWords][1] = kdeDict->getWord(i, 1);
			++numAllWords;
		}
	}
*/
	logger->append(tr2i18n("Found ").setNum(numAllWords));

	if (numAllWords == 0)
	{
		QString message = tr2i18n("Word '") + lineEdit->displayText() + tr2i18n("' was not found in the dictionary.");
		QMessageBox::information(this, tr2i18n("NK Recnik information"), message);
	}
	else
	{
		// sort them
// 		sortWords(numAllWords);
		// show them
		if (numAllWords > 100) numAllWords = 100; //limitations
		for (int i = 0; i < numAllWords; i++)
		{
			table->showRow(i);
			table->setText(i, 0, allWords[i][0]);
			table->setText(i, 1, allWords[i][1]);
		}
		tabSize = table->size();
		table->setColumnWidth(0, (tabSize.width() / 2));
		table->setColumnWidth(1, (tabSize.width() / 2));

	}

	//qDebug( "Time elapsed: %d ms", t.elapsed() );
}


void NKRecnik::rbEnToogled(int state)
{
	if (state == QButton::On)
	{
		for (int i=0; i < 100; i++)
		{
			table->setText(i, 0, "");
			table->setText(i, 1, "");
			table->hideRow(i);
		}
		Dict->dictChange(EN2SR);
// 		updateDict->dictChange(EN2SR);
		if (addDictLoaded) additionalDict->dictChange(EN2SR);
		table->horizontalHeader()->setLabel(0, tr2i18n("English"));
		table->horizontalHeader()->setLabel(1, tr2i18n("Serbian"));
		rbSr->setChecked(false);
	}
}


void NKRecnik::rbSrToogled(int state)
{
	if (state == QButton::On)
	{
		for (int i=0; i < 100; i++)
		{
			table->setText(i, 0, "");
			table->setText(i, 1, "");
			table->hideRow(i);
		}
		Dict->dictChange(SR2EN);
// 		updateDict->dictChange(SR2EN);
		if (addDictLoaded) additionalDict->dictChange(SR2EN);
		table->horizontalHeader()->setLabel(0, tr2i18n("Serbian"));
		table->horizontalHeader()->setLabel(1, tr2i18n("English"));
		rbEn->setChecked(false);
	}
}


// Escape key accelerator
void NKRecnik::keyPressEvent(QKeyEvent *k)
{
	if (k->key() == Key_Escape)
	{
		lineEdit->clear();
		lineEdit->setFocus();
		k->accept(); // Keep it here.
	}
}


void NKRecnik::tableClicked(int row, int col, int button, const QPoint & mousePos)
{
	if (button == 2 && !mousePos.isNull())
	{
		curRow = row;
		curCol = col;
		QPopupMenu* contextMenu = new QPopupMenu( this );
		Q_CHECK_PTR( contextMenu );
		QLabel *caption = new QLabel( tr2i18n("<font color=darkblue><u><b>"
			"Context Menu</b></u></font>"), this );
		caption->setAlignment( Qt::AlignCenter );
		contextMenu->insertItem(caption);
		contextMenu->insertItem(tr2i18n("&Copy"),  this, SLOT(copy()), CTRL+Key_C);
		contextMenu->insertItem(tr2i18n("&Inverse search"), this, SLOT(inverse()), CTRL+Key_I);
		contextMenu->exec(QCursor::pos());
		delete contextMenu;
	}
}


void NKRecnik::copy()
{
	QString text;
	if (clipTrans->isChecked())
		text = table->text(curRow, 1);
	else
		text = lineFormat->text().arg(table->text(curRow, 0)).arg(table->text(curRow, 1));
	clipBoard->setText(text);
}


void NKRecnik::inverse()
{
	QString word = table->text(curRow, 1);
	lineEdit->clear();
	lineEdit->setText(word);
	if (rbSr->isOn())
	{
		rbSr->setChecked(false);
		rbEn->setChecked(true);
		Dict->dictChange(EN2SR);
// 		updateDict->dictChange(EN2SR);
// 		if (addDictLoaded) additionalDict->dictChange(EN2SR);
		table->horizontalHeader()->setLabel(0, tr2i18n("English"));
		table->horizontalHeader()->setLabel(1, tr2i18n("Serbian"));
	}
	else
	{
		rbEn->setChecked(false);
		rbSr->setChecked(true);
		Dict->dictChange(SR2EN);
// 		updateDict->dictChange(SR2EN);
// 		if (addDictLoaded) additionalDict->dictChange(SR2EN);
		table->horizontalHeader()->setLabel(0, tr2i18n("Serbian"));
		table->horizontalHeader()->setLabel(1, tr2i18n("English"));
	}
	retPressed();
}


// @todo Sorting is quite slow
void NKRecnik::sortWords(int numMax)
{
	countWords(numMax);

	// Sort with number of words
	for (int i = 0; i < numMax; i++)
		for (int j = i + 1; j < numMax; j++)
			if (countAllWords[i] > countAllWords[j])
				swapWords(i, j);

	// Sort alphabetically
	for (int i = 0; i < numMax; i++)
	{
		for (int j = i + 1; j < numMax; j++)
		{
			if (countAllWords[i] == countAllWords[j])
			{
				if (allWords[i][0] > allWords[j][0])
					swapWords(i, j);
				else if (allWords[i][0] == allWords[j][0])
					if (allWords[i][1] > allWords[j][1])
						swapWords(i, j);
			}
		}
	}
}


void NKRecnik::countWords(int numMax)
{
	int spaces;
	for (int i = 0; i < numMax; i++)
	{
		spaces = 0;
		for (unsigned int j = 0; j < allWords[i][0].length(); j++)
			if (allWords[i][0][j] == ' ')
				++spaces;
		countAllWords[i] = ++spaces;
	}
}


void NKRecnik::swapWords(int index1, int index2)
{
	QString tempW;
	int tempC;

	tempW = allWords[index1][0];
	allWords[index1][0] = allWords[index2][0];
	allWords[index2][0] = tempW;

	tempW = allWords[index1][1];
	allWords[index1][1] = allWords[index2][1];
	allWords[index2][1] = tempW;

	tempC = countAllWords[index1];
	countAllWords[index1] = countAllWords[index2];
	countAllWords[index2] = tempC;
}


void NKRecnik::applySetClicked()
{
	// Set table's column width
	table->setColumnWidth(0, (tabSize.width() / 2) - widthLeft->value());
	table->setColumnWidth(1, (tabSize.width() / 2) - widthRight->value());

	// Load Additional dictionary
/*	if (addDictLoaded) delete additionalDict;
 	if (lineAddDict->text() != "")
	{
		additionalDict = new McDict(lineAddDict->text(), PLAIN_STYLE);
		additionalDict->setSort(false);
		addDictLoaded = true;
	}
	else
		addDictLoaded = false;
*/
}


void NKRecnik::saveSetClicked()
{
	tabSize = table->size();
	config->win_x = this->x();
	config->win_y = this->y();
	config->tab_left = (tabSize.width() / 2) - table->columnWidth(0);
	config->tab_right = (tabSize.width() / 2) - table->columnWidth(1);
	if (clipTrans->isChecked())
	{
		config->clip_store_trans = 1;
		config->clip_store_format = 0;
	}
	else
	{
		config->clip_store_trans = 0;
		config->clip_store_format = 1;
	}
	config->clip_format = lineFormat->text();
	config->kde_mode = 0;
// 	config->add_dict = lineAddDict->text();

	config->saveSettings();
}


void NKRecnik::helpSetClicked()
{
	QMessageBox::information(this, "Clipboard Format",
		tr2i18n("This is the format for copying.\n\n"
		"%1 is search phrase.\n"
		"%2 is translated phrase.\n\n"
		"So, you could put something like, 'phrase %1 means %2 in English'."));
}


void NKRecnik::browseSetClicked()
{
	/*QString addDictFile;
	addDictFile = KFileDialog::getOpenFileName(config->homeDir(), "*.dat | Dictionary file (*.dat)\n", this, "Open Dictionary");
	logger->append("Opened file: " + addDictFile);
	lineAddDict->setText(addDictFile);*/
}


void NKRecnik::updateClicked()
{
	/*if(!updateDict->writeEntry(upEnLine->text(), upSrLine->text()))
		QMessageBox::warning(this, tr2i18n("Failed!"), tr2i18n("For some reason, updating was not performed!"),0,0,1);
	else
	{
		logger->append("Ok, " + upEnLine->text() + " added.");
		upSrLine->setText("");
		upEnLine->setText("");
		upEnLine->setFocus();
	}*/
}


void NKRecnik::setFormatToggled(bool on)
{
	if (on)
		clipTrans->setChecked(false);
	else
		clipTrans->setChecked(true);
}


void NKRecnik::setStoreToggled(bool on)
{
	if (on)
		clipFormat->setChecked(false);
	else
		clipFormat->setChecked(true);
}


/*void NKRecnik::focusInEvent(QFocusEvent *ev)
{
	qDebug ("I GOT FOCUS!");
	lineEdit->setFocus();
}*/

#include "nkrecnik.moc"
